@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
 #
 # Copyright 2006-2011 Zuza Software Foundation
 #
@@ -20,17 +18,16 @@
 
 """Parent class for LISA standards (TMX, TBX, XLIFF)"""
 
-import re
-
-try:
-    from lxml import etree
-    from translate.misc.xml_helpers import getText, getXMLlang, setXMLlang, \
-                                           getXMLspace, setXMLspace, namespaced
-except ImportError, e:
-    raise ImportError("lxml is not installed. It might be possible to continue without support for XML formats.")
+from lxml import etree
 
+from translate.misc.xml_helpers import (
+    getText,
+    getXMLlang,
+    getXMLspace,
+    namespaced,
+    reindent,
+)
 from translate.storage import base
-from translate.lang import data
 
 
 class LISAunit(base.TranslationUnit):
@@ -39,12 +36,12 @@ class LISAunit(base.TranslationUnit):
     languages possible.
     """
 
-    #The name of the root element of this unit type:(termEntry, tu, trans-unit)
+    # The name of the root element of this unit type:(termEntry, tu, trans-unit)
     rootNode = ""
     # The name of the per language element of this unit type:(termEntry, tu,
     # trans-unit)
     languageNode = ""
-    #The name of the innermost element of this unit type:(term, seg)
+    # The name of the innermost element of this unit type:(term, seg)
     textNode = ""
 
     namespace = None
@@ -62,26 +59,27 @@ class LISAunit(base.TranslationUnit):
             self._state_n = 0
             return
         self.xmlelement = etree.Element(self.namespaced(self.rootNode))
-        #add descrip, note, etc.
-        super(LISAunit, self).__init__(source)
+        # add descrip, note, etc.
+        super().__init__(source)
 
     def __eq__(self, other):
         """Compares two units"""
         if not isinstance(other, LISAunit):
-            return super(LISAunit, self).__eq__(other)
+            return super().__eq__(other)
         languageNodes = self.getlanguageNodes()
         otherlanguageNodes = other.getlanguageNodes()
         if len(languageNodes) != len(otherlanguageNodes):
             return False
         for i in range(len(languageNodes)):
-            mytext = self.getNodeText(languageNodes[i],
-                                      getXMLspace(self.xmlelement,
-                                                  self._default_xml_space))
-            othertext = other.getNodeText(otherlanguageNodes[i],
-                                          getXMLspace(self.xmlelement,
-                                                      self._default_xml_space))
+            mytext = self.getNodeText(
+                languageNodes[i], getXMLspace(self.xmlelement, self._default_xml_space)
+            )
+            othertext = other.getNodeText(
+                otherlanguageNodes[i],
+                getXMLspace(self.xmlelement, self._default_xml_space),
+            )
             if mytext != othertext:
-                #TODO:^ maybe we want to take children and notes into account
+                # TODO:^ maybe we want to take children and notes into account
                 return False
         return True
 
@@ -106,23 +104,25 @@ class LISAunit(base.TranslationUnit):
 
     def get_source_dom(self):
         return self.getlanguageNode(lang=None, index=0)
+
     source_dom = property(get_source_dom, set_source_dom)
 
-    def setsource(self, text, sourcelang='en'):
-        if self._rich_source is not None:
-            self._rich_source = None
-        text = data.forceunicode(text)
-        self.source_dom = self.createlanguageNode(sourcelang, text, "source")
+    @property
+    def source(self):
+        return self.getNodeText(
+            self.source_dom, getXMLspace(self.xmlelement, self._default_xml_space)
+        )
 
-    def getsource(self):
-        return self.getNodeText(self.source_dom,
-                                getXMLspace(self.xmlelement,
-                                            self._default_xml_space))
-    source = property(getsource, setsource)
+    @source.setter
+    def source(self, source):
+        self.setsource(source, sourcelang="en")
+
+    def setsource(self, text, sourcelang="en"):
+        self._rich_source = None
+        self.source_dom = self.createlanguageNode(sourcelang, text, "source")
 
     def set_target_dom(self, dom_node, append=False):
         languageNodes = self.getlanguageNodes()
-        assert len(languageNodes) > 0
         if dom_node is not None:
             if append or len(languageNodes) == 0:
                 self.xmlelement.append(dom_node)
@@ -136,81 +136,62 @@ class LISAunit(base.TranslationUnit):
             return self.getlanguageNode(lang=lang)
         else:
             return self.getlanguageNode(lang=None, index=1)
+
     target_dom = property(get_target_dom)
 
-    def settarget(self, text, lang='xx', append=False):
-        """Sets the "target" string (second language), or alternatively
-        appends to the list"""
-        #XXX: we really need the language - can't really be optional, and we
+    def gettarget(self, lang=None):
+        """retrieves the "target" text (second entry), or the entry in the
+        specified language, if it exists
+        """
+        return self.getNodeText(
+            self.get_target_dom(lang),
+            getXMLspace(self.xmlelement, self._default_xml_space),
+        )
+
+    def settarget(self, target, lang="xx", append=False):
+        """Sets the "target" string (second language), or alternatively appends
+        to the list
+        """
+        # XXX: we really need the language - can't really be optional, and we
         # need to propagate it
         if self._rich_target is not None:
             self._rich_target = None
-        text = data.forceunicode(text)
         # Firstly deal with reinitialising to None or setting to identical
         # string
-        if self.gettarget() == text:
+        if self.target == target:
             return
-        languageNode = self.get_target_dom(None)
-        if not text is None:
+        languageNode = self.target_dom
+        if target is not None:
             if languageNode is None:
-                languageNode = self.createlanguageNode(lang, text, "target")
+                languageNode = self.createlanguageNode(lang, target, "target")
                 self.set_target_dom(languageNode, append)
             else:
                 if self.textNode:
                     terms = languageNode.iter(self.namespaced(self.textNode))
                     try:
-                        languageNode = terms.next()
-                    except StopIteration, e:
+                        languageNode = next(terms)
+                    except StopIteration:
                         pass
-                languageNode.text = text
+                languageNode.text = target
         else:
             self.set_target_dom(None, False)
 
-    def gettarget(self, lang=None):
-        """retrieves the "target" text (second entry), or the entry in the
-        specified language, if it exists"""
-        return self.getNodeText(self.get_target_dom(lang),
-                                getXMLspace(self.xmlelement,
-                                            self._default_xml_space))
-    target = property(gettarget, settarget)
+    @property
+    def target(self):
+        return self.gettarget()
+
+    @target.setter
+    def target(self, target):
+        self.settarget(target)
 
     def createlanguageNode(self, lang, text, purpose=None):
         """Returns a xml Element setup with given parameters to represent a
-        single language entry. Has to be overridden."""
+        single language entry. Has to be overridden.
+        """
         return None
 
-    def createPHnodes(self, parent, text):
-        """Create the text node in parent containing all the ph tags"""
-        matches = _getPhMatches(text)
-        if not matches:
-            parent.text = text
-            return
-
-        # Now we know there will definitely be some ph tags
-        start = matches[0].start()
-        pretext = text[:start]
-        if pretext:
-            parent.text = pretext
-        lasttag = parent
-        for i, m in enumerate(matches):
-            #pretext
-            pretext = text[start:m.start()]
-            # this will never happen with the first ph tag
-            if pretext:
-                lasttag.tail = pretext
-            #ph node
-            phnode = etree.SubElement(parent, self.namespaced("ph"))
-            phnode.set("id", str(i + 1))
-            phnode.text = m.group()
-            lasttag = phnode
-            start = m.end()
-        #post text
-        if text[start:]:
-            lasttag.tail = text[start:]
-
     def getlanguageNodes(self):
-        """Returns a list of all nodes that contain per language information.
-        """
+        """Returns a list of all nodes that contain per language information."""
         return list(self.xmlelement.iterchildren(self.namespaced(self.languageNode)))
 
     def getlanguageNode(self, lang=None, index=None):
@@ -237,60 +218,69 @@ class LISAunit(base.TranslationUnit):
             terms = languageNode.iterdescendants(self.namespaced(self.textNode))
             if terms is None:
                 return None
-            try:
-                return getText(terms.next(), xml_space)
-            except StopIteration:
-                # didn't have the structure we expected
-                return None
+            node = next(terms, None)
+            if node is not None:
+                return getText(node, xml_space)
+            # didn't have the structure we expected
+            return None
         else:
             return getText(languageNode, xml_space)
 
     def __str__(self):
-        return etree.tostring(self.xmlelement, pretty_print=True,
-                              encoding='utf-8')
+        # 'unicode' encoding keeps the unicode status of the output
+        return etree.tostring(self.xmlelement, pretty_print=True, encoding="unicode")
 
     def _set_property(self, name, value):
         self.xmlelement.attrib[name] = value
 
-    xid = property(lambda self: self.xmlelement.attrib[self.namespaced('xid')],
-                   lambda self, value: self._set_property(self.namespaced('xid'), value))
+    xid = property(
+        lambda self: self.xmlelement.attrib[self.namespaced("xid")],
+        lambda self, value: self._set_property(self.namespaced("xid"), value),
+    )
 
-    rid = property(lambda self: self.xmlelement.attrib[self.namespaced('rid')],
-                   lambda self, value: self._set_property(self.namespaced('rid'), value))
+    rid = property(
+        lambda self: self.xmlelement.attrib[self.namespaced("rid")],
+        lambda self, value: self._set_property(self.namespaced("rid"), value),
+    )
 
+    @classmethod
     def createfromxmlElement(cls, element):
         term = cls(None, empty=True)
         term.xmlelement = element
         return term
-    createfromxmlElement = classmethod(createfromxmlElement)
 
 
 class LISAfile(base.TranslationStore):
     """A class representing a file store for one of the LISA file formats."""
+
     UnitClass = LISAunit
-    #The root node of the XML document:
+    # The root node of the XML document:
     rootNode = ""
-    #The root node of the content section:
+    # The root node of the content section:
     bodyNode = ""
-    #The XML skeleton to use for empty construction:
+    # The XML skeleton to use for empty construction:
     XMLskeleton = ""
+    XMLindent = {}
+    XMLdoublequotes = True
+    XMLdoctype = None
+    XMLuppercaseEncoding = True
 
     namespace = None
 
-    def __init__(self, inputfile=None, sourcelanguage='en',
-                 targetlanguage=None, unitclass=None):
-        super(LISAfile, self).__init__(unitclass=unitclass)
+    def __init__(
+        self, inputfile=None, sourcelanguage="en", targetlanguage=None, **kwargs
+    ):
+        super().__init__(**kwargs)
         if inputfile is not None:
             self.parse(inputfile)
             assert self.document.getroot().tag == self.namespaced(self.rootNode)
         else:
             # We strip out newlines to ensure that spaces in the skeleton
             # doesn't interfere with the the pretty printing of lxml
-            self.parse(self.XMLskeleton.replace("\n", ""))
+            self.parse(self.XMLskeleton.replace("\n", "").encode("utf-8"))
             self.setsourcelanguage(sourcelanguage)
             self.settargetlanguage(targetlanguage)
             self.addheader()
-        self._encoding = "UTF-8"
 
     def addheader(self):
         """Method to be overridden to initialise headers, etc."""
@@ -309,10 +299,11 @@ class LISAfile(base.TranslationStore):
         return namespaced(self.namespace, name)
 
     def initbody(self):
-        """Initialises self.body so it never needs to be retrieved from the
-        XML again."""
+        """Initialises self.body so it never needs to be retrieved from the XML
+        again.
+        """
         self.namespace = self.document.getroot().nsmap.get(None, None)
-        self.body = self.document.find('//%s' % self.namespaced(self.bodyNode))
+        self.body = self.document.find("//%s" % self.namespaced(self.bodyNode))
 
     def addsourceunit(self, source):
         """Adds and returns a new unit with the given string as first entry."""
@@ -322,33 +313,61 @@ class LISAfile(base.TranslationStore):
 
     def addunit(self, unit, new=True):
         unit.namespace = self.namespace
-        super(LISAfile, self).addunit(unit)
+        super().addunit(unit)
         if new:
             self.body.append(unit.xmlelement)
 
-    def __str__(self):
+    def removeunit(self, unit):
+        super().removeunit(unit)
+        unit.xmlelement.getparent().remove(unit.xmlelement)
+
+    def serialize_hook(self, treestring):
+        return treestring
+
+    def serialize(self, out=None):
         """Converts to a string containing the file's XML"""
-        return etree.tostring(self.document, pretty_print=True,
-                              xml_declaration=True, encoding='utf-8')
+        root = self.document.getroot()
+        if self.XMLdoublequotes:
+            if self.XMLuppercaseEncoding:
+                out.write(b'<?xml version="1.0" encoding="UTF-8"?>\n')
+            else:
+                out.write(b'<?xml version="1.0" encoding="utf-8"?>\n')
+        if self.XMLindent:
+            reindent(root, **self.XMLindent)
+        if 1:
+            treestring = etree.tostring(
+                self.document,
+                pretty_print=not self.XMLindent,
+                xml_declaration=not self.XMLdoublequotes,
+                encoding=self.encoding,
+                doctype=self.XMLdoctype,
+            )
+            treestring = self.serialize_hook(treestring)
+            out.write(treestring)
+            return
+        self.document.write(
+            out,
+            pretty_print=not self.XMLindent,
+            xml_declaration=not self.XMLdoublequotes,
+            encoding=("UTF-8" if self.XMLuppercaseEncoding else "utf-8"),
+            doctype=self.XMLdoctype,
+        )
 
     def parse(self, xml):
         """Populates this object from the given xml string"""
-        if not hasattr(self, 'filename'):
-            self.filename = getattr(xml, 'name', '')
+        if not hasattr(self, "filename"):
+            self.filename = getattr(xml, "name", "")
         if hasattr(xml, "read"):
             xml.seek(0)
             posrc = xml.read()
             xml = posrc
-        if etree.LXML_VERSION >= (2, 1, 0):
-            #Since version 2.1.0 we can pass the strip_cdata parameter to
-            #indicate that we don't want cdata to be converted to raw XML
-            parser = etree.XMLParser(strip_cdata=False)
-        else:
-            parser = etree.XMLParser()
+        parser = etree.XMLParser(strip_cdata=False, resolve_entities=False)
         self.document = etree.fromstring(xml, parser).getroottree()
-        self._encoding = self.document.docinfo.encoding
+        self.encoding = self.document.docinfo.encoding
         self.initbody()
         assert self.document.getroot().tag == self.namespaced(self.rootNode)
-        for entry in self.document.getroot().iterdescendants(self.namespaced(self.UnitClass.rootNode)):
+        for entry in self.document.getroot().iterdescendants(
+            self.namespaced(self.UnitClass.rootNode)
+        ):
             term = self.UnitClass.createfromxmlElement(entry)
             self.addunit(term, new=False)
