@@ -136,6 +136,13 @@ class TestJSONResourceStore(test_monolingual.TestMonolingualStore):
 
         assert out.getvalue() == b'{\n    "key": "value"\n}\n'
 
+    def test_can_not_detect(self):
+        store = self.StoreClass()
+        with raises(base.ParseError):
+            store.parse(
+                b"PK\x03\x04\x14\x00\x06\x00\x08\x00\x00\x00!\x00b\xee\x9dh^\x01\x00\x00\x90\x04\x00\x00\x13\x00\x08\x02"
+            )
+
     def test_error(self):
         store = self.StoreClass()
         with raises(base.ParseError):
@@ -312,6 +319,32 @@ class TestJSONNestedResourceStore(test_monolingual.TestMonolingualUnit):
 """
         assert bytes(store).decode() == expected
 
+    def test_list_to_dict(self):
+        data = """{
+    "userInfoPage": [
+        "Name"
+    ]
+}
+"""
+        store = self.StoreClass()
+        store.parse(data)
+        assert len(store.units) == 1
+        assert bytes(store).decode() == data
+
+        unit = self.StoreClass.UnitClass("Test")
+        unit.setid("userInfoPage.nesting")
+        store.addunit(unit)
+        assert (
+            bytes(store).decode()
+            == """{
+    "userInfoPage": {
+        "0": "Name",
+        "nesting": "Test"
+    }
+}
+"""
+        )
+
 
 class TestWebExtensionUnit(test_monolingual.TestMonolingualUnit):
     UnitClass = jsonl10n.WebExtensionJsonUnit
